# -*- encoding: utf-8 -*-
"""Generate checkerboard wallpaper images.

Usage:
  specktre.py new --size=<size> --start=<start> --end=<end> [--squares | --triangles | --hexagons] [--name=<name>]
  specktre.py -h

Options:
  -h --help          Show this screen.
  --size=<size>      Size in pixels - WxH (e.g. 100x200)
  --start=<start>    Start of the color range (hex, e.g. #01ab23)
  --end=<end>        End of the color range (hex, e.g. #01ab23)
  --squares          Tile with squares.
  --triangles        Tile with triangles.
  --hexagons         Tile with hexagons.
  --name=<name>      (Optional) Name of the file to save to.

"""  # noqa

import re
import sys

import attr
import docopt

from .colors import RGBColor
from .tilings import generate_hexagons, generate_squares, generate_triangles


@attr.s
class Settings(object):
    generator = attr.ib()
    width = attr.ib()
    height = attr.ib()
    start_color = attr.ib()
    end_color = attr.ib()
    name = attr.ib()


def check_positive_integer(name, value):
    """Check a value is a positive integer.

    Returns the value if so, raises ValueError otherwise.

    """
    try:
        value = int(value)
        is_positive = (value > 0)
    except ValueError:
        raise ValueError('%s should be an integer; got %r' % (name, value))

    if is_positive:
        return value
    else:
        raise ValueError('%s should be positive; got %r' % (name, value))


def check_color_input(value):
    """Check a value is a valid colour input.

    Returns a parsed `RGBColor` instance if so, raises ValueError
    otherwise.

    """
    value = value.lower()
    # Trim a leading hash
    if value.startswith('#'):
        value = value[1:]

    if len(value) != 6:
        raise ValueError(
            'Color should be six hexadecimal digits, got %r (%s)' %
            (value, len(value)))

    if re.sub(r'[a-f0-9]', '', value):
        raise ValueError(
            'Color should only contain hex characters, got %r' % value)

    red = int(value[0:2], base=16)
    green = int(value[2:4], base=16)
    blue = int(value[4:6], base=16)
    return RGBColor(red, green, blue)


def parse_args(argv):
    args = docopt.docopt(__doc__, argv)

    if args['--squares']:
        generator = generate_squares
    elif args['--triangles']:
        generator = generate_triangles
    elif args['--hexagons']:
        generator = generate_hexagons
    else:
        generator = generate_squares

    try:
        width, height = args['--size'].split('x')
        width = check_positive_integer(name='Width', value=width)
        height = check_positive_integer(name='Height', value=height)
    except ValueError:
        sys.exit('--size should be in the form WxH; got %s' % args['--size'])

    start_color = check_color_input(args['--start'])
    end_color = check_color_input(args['--end'])

    name = args['--name']

    return Settings(
        generator=generator,
        width=width,
        height=height,
        start_color=start_color,
        end_color=end_color,
        name=name,
    )



# -*- encoding: utf-8 -*-
"""Generate random colors between two other colors."""

import random

import attr


@attr.s
class RGBColor(object):
    red = attr.ib()
    green = attr.ib()
    blue = attr.ib()


def random_color(start, end):
    d_red = (start.red - end.red)
    d_green = (start.green - end.green)
    d_blue = (start.blue - end.blue)
    while True:
        chosen_d = random.uniform(0, 1)
        yield RGBColor(
            start.red - int(d_red * chosen_d),
            start.green - int(d_green * chosen_d),
            start.blue - int(d_blue * chosen_d)
        )



# -*- encoding: utf-8 -*-
"""Utility functions."""

import os
import random
import string


def _candidate_filenames():
    """Generates filenames of the form 'specktre_123AB.png'.

    The random noise is five characters long, which allows for
    62^5 = 916 million possible filenames.

    """
    while True:
        random_stub = ''.join([
            random.choice(string.ascii_letters + string.digits)
            for _ in range(5)
        ])
        yield 'specktre_%s.png' % random_stub


def new_filename():
    """Returns a filename for a new specktre image.

    This filename is of the form 'specktre_123AB.png' and does not
    already exist when this function is called.

    """
    for filename in _candidate_filenames():
        if not os.path.exists(filename):
            return filename

    # Since _candidate_filenames() is an infinite generator and produces
    # millions of options, this branch should never be hit in practice.
    # Keeps coverage happy.
    else:
        assert False, 'Should not be reachable'



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""Generate coordinates for tilings of the plane.

Diagrams and background to this code are on my blog:
http://alexwlchan.net/2016/10/tiling-the-plane-with-pillow/

"""

from __future__ import division

import math


def _scale_coordinates(generator, image_width, image_height, side_length=50):
    scaled_width = int(image_width / side_length) + 1
    scaled_height = int(image_height / side_length) + 1

    for coords in generator(scaled_width, scaled_height):
        yield [(x * side_length, y * side_length) for (x, y) in coords]


def generate_unit_squares(image_width, image_height):
    """Generate coordinates for a tiling of unit squares."""
    # Iterate over the required rows and cells.  The for loops (x, y)
    # give the coordinates of the top left-hand corner of each square:
    #
    #      (x, y) +-----+ (x + 1, y)
    #             |     |
    #             |     |
    #             |     |
    #  (x, y + 1) +-----+ (x + 1, y + 1)
    #
    for x in range(image_width):
        for y in range(image_height):
            yield [(x, y), (x + 1, y), (x + 1, y + 1), (x, y + 1)]


def generate_squares(*args, **kwargs):
    """Generate coordinates for a tiling of squares."""
    return _scale_coordinates(generate_unit_squares, *args, **kwargs)


def generate_unit_triangles(image_width, image_height):
    """Generate coordinates for a tiling of unit triangles."""
    # Our triangles lie with one side parallel to the x-axis.  Let s be
    # the length of one side, and h the height of the triangle.
    #
    # The for loops (x, y) gives the coordinates of the top left-hand corner
    # of a pair of triangles:
    #
    #           (x, y) +-----+ (x + 1, y)
    #                   \   / \
    #                    \ /   \
    #    (x + 1/2, y + h) +-----+ (x + 3/2, y + h)
    #
    # where h = sin(60°) is the height of an equilateral triangle with
    # side length 1.
    #
    # On odd-numbered rows, we translate by (s/2, 0) to make the triangles
    # line up with the even-numbered rows.
    #
    # To avoid blank spaces on the edge of the canvas, the first pair of
    # triangles on each row starts at (-1, 0) -- one width before the edge
    # of the canvas.
    h = math.sin(math.pi / 3)

    for x in range(-1, image_width):
        for y in range(int(image_height / h)):

            # Add a horizontal offset on odd numbered rows
            x_ = x if (y % 2 == 0) else x + 0.5

            yield [(x_, y * h), (x_ + 1, y * h), (x_ + 0.5, (y + 1) * h)]
            yield [(x_ + 1, y * h), (x_ + 1.5, (y + 1) * h),
                   (x_ + 0.5, (y + 1) * h)]


def generate_triangles(*args, **kwargs):
    """Generate coordinates for a tiling of triangles."""
    return _scale_coordinates(generate_unit_triangles, *args, **kwargs)


def generate_unit_hexagons(image_width, image_height):
    """Generate coordinates for a regular tiling of unit hexagons."""
    # Let s be the length of one side of the hexagon, and h the height
    # of the entire hexagon if one side lies parallel to the x-axis.
    #
    # The for loops (x, y) give the coordinate of one coordinate of the
    # hexagon, and the remaining coordinates fall out as follows:
    #
    #                     (x, y) +-----+ (x + 1, y)
    #                           /       \
    #                          /         \
    #         (x - 1/2 y + h) +           + (x + 3/2, y + h)
    #                          \         /
    #                           \       /
    #                 (x, y + 2h) +-----+ (x + 1, y + 2h)
    #
    # In each row we generate hexagons in the following pattern
    #
    #         /‾‾‾\   /‾‾‾\   /‾‾‾\
    #         \___/   \___/   \___/
    #
    # and the next row is offset to fill in the gaps. So after two rows,
    # we'd have the following pattern:
    #
    #         /‾‾‾\   /‾‾‾\   /‾‾‾\
    #         \___/‾‾‾\___/‾‾‾\___/‾‾‾\
    #             \___/   \___/   \___/
    #
    # There are offsets to ensure we fill the entire canvas.

    # Half the height of the hexagon
    h = math.sin(math.pi / 3)

    for x in range(-1, image_width, 3):
        for y in range(-1, int(image_height / h) + 1):

            # Add the horizontal offset on every other row
            x_ = x if (y % 2 == 0) else x + 1.5

            yield [
                (x_,        y * h),
                (x_ + 1,    y * h),
                (x_ + 1.5, (y + 1) * h),
                (x_ + 1,   (y + 2) * h),
                (x_,       (y + 2) * h),
                (x_ - 0.5, (y + 1) * h),
            ]


def generate_hexagons(*args, **kwargs):
    """Generate coordinates for a tiling of hexagons."""
    return _scale_coordinates(generate_unit_hexagons, *args, **kwargs)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-

import codecs
import os

from setuptools import find_packages, setup


def local_file(name):
    return os.path.relpath(os.path.join(os.path.dirname(__file__), name))


SOURCE = local_file('src')
README = local_file('README.rst')
long_description = codecs.open(README, encoding='utf-8').read()

# For PyPI purposes, change the image link to point to GitHub
long_description = long_description.replace(
    'output/demo.png',
    'https://github.com/alexwlchan/specktre/raw/master/output/demo.png')


setup(
    name='specktre',
    version='0.3.0',
    description='A tool for creating wallpapers with Python',
    long_description=long_description,
    url='https://github.com/alexwlchan/specktre',
    author='Alex Chan',
    author_email='alex@alexwlchan.net',
    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Other Audience',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    keywords='images wallpaper',
    packages=find_packages(SOURCE),
    package_dir={'': SOURCE},
    install_requires=[
        "attrs>=19.1.0,<20",
        'docopt',
        'Pillow',
    ],

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    entry_points={
        'console_scripts': [
            'specktre=specktre:main',
        ],
    },
)



# -*- encoding: utf-8 -*-

import sys

from PIL import Image, ImageDraw

from . import cli
from .colors import random_color
from .utils import new_filename


def draw_speckled_wallpaper(settings):
    im = Image.new(mode='RGB', size=(settings.width, settings.height))
    squares = settings.generator(settings.width, settings.height)
    colors = random_color(settings.start_color, settings.end_color)
    for sq, color in zip(squares, colors):
        ImageDraw.Draw(im).polygon(sq, fill=color)

    return im


def save_speckled_wallpaper(settings):
    im = draw_speckled_wallpaper(settings)
    if settings.name:
        filename = settings.name
    else:
        filename = new_filename()
    im.save(filename)
    print('Saved new wallpaper as %s' % filename)


def main():
    settings = cli.parse_args(sys.argv)
    save_speckled_wallpaper(settings)
